@@ -21,8 +21,8 @@
  ============================================================================
  Name        : main.c
  Author      : smurfy
- Version     : 0.0.3
- Description : Fahrplan application for german trains
+ Version     : 0.0.4
+ Description : Fahrplan application for train lines in Europe
  ============================================================================
  */
 /* Includes */
@@ -47,7 +47,7 @@
 #include "localisation.h"
 
 #define APP_NAME "fahrplan"
-#define APP_VER "0.0.3"
+#define APP_VER "0.0.4"
 #define APP_SERVICE "com.nokia.fahrplan"
 #define APP_METHOD "/com/nokia/fahrplan"
 /* end defines */
@@ -82,11 +82,11 @@ struct _AppData {
 	GtkWidget *currentTmpPicker;
 	GtkWidget *currentSelectorFor;
 	GtkWidget *currentResultPannableArea;
-	GtkWidget *currentResultSelector;
-	GtkListStore   *currentResultModelStore;
+	GtkWidget *currentResultMainVbox;
 	GtkWidget *currentDetailsPannableArea;
-	GtkWidget *currentDetailsSelector;
-	GtkListStore   *currentDetailsModelStore;
+	GtkWidget *currentDetailsMainVbox;
+
+	LocationGPSDControl *currentGPSControl;
 
 	char currentResultBaseUrl[255];
 };
@@ -101,6 +101,34 @@ struct CURLData {
 	size_t size;
 };
 
+/**
+ * Simple String replace function
+ */
+
+char * str_replace( const char *string, const char *substr, const char *replacement ){
+	char *tok = NULL;
+	char *newstr = NULL;
+	tok = strstr( string, substr );
+	if( tok == NULL ) return strdup( string );
+	newstr = malloc( strlen( string ) - strlen( substr ) + strlen( replacement ) + 1 );
+	if( newstr == NULL ) return NULL;
+	memcpy( newstr, string, tok - string );
+	memcpy( newstr + (tok - string), replacement, strlen( replacement ) );
+	memcpy( newstr + (tok - string) + strlen( replacement ), tok + strlen( substr ), strlen( string ) - strlen( substr ) - ( tok - string ) );
+	memset( newstr + strlen( string ) - strlen( substr ) + strlen( replacement ), 0, 1 );
+	return newstr;
+}
+
+void removeLineBreaks(char *string) {
+
+	unsigned int i = 0;
+	for(i = 0; i < strlen(string); ++i){
+		if (string[i] == '\n'){
+			string[i] = ' ';
+		}
+	}
+}
+
 /**
  * Copys data from internal curl buffer to our own buffer.
  */
@@ -118,6 +146,16 @@ static size_t copyDataBuffer(void *ptr, size_t size, size_t nmemb, void *ctx) {
 	return size * nmemb;
 }
 
+/**
+ * Function to handle the selection of an entry of the results page
+ */
+void resultSelected(GtkButton * button, AppData *data) {
+
+	gchar *current_selection;
+	current_selection = g_object_get_data(G_OBJECT(button), "URL");
+	loadDetails(current_selection, data);
+}
+
 /**
  * Parse function for the search result. given as libxml node.
  */
@@ -247,23 +285,33 @@ static void getResultValues(xmlNode * a_node, AppData *data) {
 				sprintf(infoLabel, "<b>%s - %s</b> %s (%s %s)\n%s", fromTimeStr, toTimeStr, trainTypeStr, changeStr, transfersStr ,warningStr);
 			}
 
-			GtkTreeIter    iter;
-
-			/* Append a row and fill in some data */
-			gtk_list_store_append (data->currentResultModelStore, &iter);
-			gtk_list_store_set (data->currentResultModelStore, &iter,
-							  0, infoLabel,
-							  -1);
-
-			gint num  = gtk_tree_model_iter_n_children (data->currentResultModelStore, NULL);
-
-			sprintf(urlLabel, "URL%d", num);
-
 			gchar *trainUrlGChar;
 
 			trainUrlGChar = g_strdup_printf("%s", trainUrlStr);
 
-			g_object_set_data(G_OBJECT(data->currentResultSelector), urlLabel, trainUrlGChar);
+			GtkWidget *line;
+			GtkWidget *lineHbox;
+			GtkWidget *lineText;
+
+			line = gtk_button_new();
+			gtk_button_set_focus_on_click(GTK_BUTTON(line), FALSE);
+			gtk_button_set_relief(GTK_BUTTON(line), GTK_RELIEF_NONE);
+
+			gtk_widget_set_size_request  (line, -1, 70);
+
+			lineHbox = gtk_hbox_new(FALSE, 0);
+			gtk_container_add(GTK_CONTAINER(line), lineHbox);
+			lineText = gtk_label_new(NULL);
+			gtk_label_set_justify(GTK_LABEL(lineText), GTK_JUSTIFY_FILL);
+			gtk_label_set_markup(GTK_LABEL(lineText), infoLabel);
+			gtk_box_pack_start(GTK_BOX(lineHbox), lineText, FALSE, TRUE, 10);
+
+			gtk_box_pack_start(GTK_BOX(data->currentResultMainVbox), line, FALSE, TRUE, 0);
+			gtk_box_pack_start(GTK_BOX(data->currentResultMainVbox), gtk_hseparator_new(), FALSE, TRUE, 0);
+
+			g_signal_connect (G_OBJECT(line), "clicked",  G_CALLBACK (resultSelected), data);
+
+			g_object_set_data(G_OBJECT(line), "URL", trainUrlGChar);
 		}
 		/* g_debug("stuff: %s , %s , %s ", timeStr, changeStr, trainTypeStr); */
 	}
@@ -397,8 +445,6 @@ void earlierClicked(HildonButton * button, AppData *data) {
  */
 void loadDetails(char* targetUrl, AppData *data) {
 
-	return;
-
 	/* Start Curl search */
 	CURL *curl;
 	CURLcode res;
@@ -477,27 +523,15 @@ void loadDetails(char* targetUrl, AppData *data) {
 			gtk_window_set_title(GTK_WINDOW (data->detailsWindow),
 					_("Details"));
 
-			GtkCellRenderer     *renderer;
+			GtkWidget *line;
+			GtkWidget *lineHbox;
+			GtkWidget *lineText;
 
-			data->currentDetailsSelector = hildon_gtk_tree_view_new(HILDON_UI_MODE_EDIT);
+			data->currentDetailsMainVbox = gtk_vbox_new(FALSE, 5);
 
 			data->currentDetailsPannableArea = hildon_pannable_area_new();
 
-			renderer = gtk_cell_renderer_text_new ();
-			gtk_cell_renderer_set_fixed_size(renderer, 100, 200);
-
-			gtk_tree_view_insert_column_with_attributes (GTK_TREE_VIEW (data->currentDetailsSelector),
-													   -1,
-													   "Title",
-													   renderer,
-													   "markup", 0,
-													   "expand", 1,
-													   NULL);
-
-			gtk_tree_view_set_fixed_height_mode(GTK_TREE_VIEW (data->currentDetailsSelector), FALSE);
-
-
-			data->currentDetailsModelStore = gtk_list_store_new (1, G_TYPE_STRING);
+		    hildon_pannable_area_add_with_viewport(HILDON_PANNABLE_AREA (data->currentDetailsPannableArea), GTK_WIDGET (data->currentDetailsMainVbox));
 
 			for (cur_node = root_element->children; cur_node; cur_node
 					= cur_node->next) {
@@ -524,6 +558,7 @@ void loadDetails(char* targetUrl, AppData *data) {
 						char *infoStr = "";
 						char *toStr = "";
 						char *timeToStr = "";
+						char *alternateStr = "";
 
 						for (sub_node = cur_node->children; sub_node; sub_node
 										= sub_node->next) {
@@ -556,47 +591,49 @@ void loadDetails(char* targetUrl, AppData *data) {
 									if (!strcmp(sub_node2->name, "span") && !strcmp(trainStr, "")) {
 
 										trainStr = sub_node2->children->content;
+										trainStr = str_replace(trainStr, "     ", " ");
 									}
 								}
 							}
 
 							if (!strcmp(sub_node->name, "text")) {
 
-								char *testFrom = strstr(sub_node->content, "ab");
-								int found = 0;
+								if (strcmp(sub_node->content, "") && strcmp(sub_node->content, "\n")) {
 
-								if (testFrom) {
+									char *testFrom = strstr(sub_node->content, "ab");
+									int found = 0;
 
-									timeFromStr = testFrom;
-									timeFromStr += 3;
-									found = 1;
+									if (testFrom) {
+
+										timeFromStr = testFrom;
+										timeFromStr += 3;
+										found = 1;
+									}
+									testFrom = strstr(sub_node->content, "an");
+
+									if (testFrom) {
+
+										timeToStr = testFrom;
+										timeToStr += 3;
+										found = 1;
+									}
+
+									if (found == 0) {
+
+										alternateStr = sub_node->content;
+										alternateStr += 1;
+									}
 								}
-								testFrom = strstr(sub_node->content, "an");
-
-								if (testFrom) {
-
-									timeToStr = testFrom;
-									timeToStr += 3;
-									found = 1;
-								}
-
-								if (found == 0) {
-
-									timeFromStr = sub_node->content;
-								}
-
 							}
 						}
 
-						GtkTreeIter    iter;
-						char infoLabel[1000];
-						sprintf(infoLabel, "%s %s %s %s -%s", fromStr, timeFromStr, toStr, timeToStr, trainStr);
-
-						/* Append a row and fill in some data */
-						gtk_list_store_append (data->currentDetailsModelStore, &iter);
-						gtk_list_store_set (data->currentDetailsModelStore, &iter,
-										  0, infoLabel,
-										  -1);
+						removeLineBreaks(fromStr);
+						removeLineBreaks(timeFromStr);
+						removeLineBreaks(toStr);
+						removeLineBreaks(timeToStr);
+						removeLineBreaks(trainStr);
+						removeLineBreaks(infoStr);
+						removeLineBreaks(alternateStr);
 
 						g_debug("From: %s", fromStr);
 						g_debug("FromTime: %s", timeFromStr);
@@ -604,6 +641,64 @@ void loadDetails(char* targetUrl, AppData *data) {
 						g_debug("ToTime: %s", timeToStr);
 						g_debug("Train: %s", trainStr);
 						g_debug("Info: %s", infoStr);
+						g_debug("Alt: %s", alternateStr);
+
+						GtkTreeIter    iter;
+						char infoLabel[1000] = "";
+
+						if (strcmp(trainStr,"")) {
+
+							strcat(infoLabel, "<b>");
+							strcat(infoLabel, trainStr);
+							strcat(infoLabel, "</b>\n");
+						}
+
+						strcat(infoLabel, fromStr);
+						if (strcmp(timeFromStr,"")) {
+
+							strcat(infoLabel, " - ");
+							strcat(infoLabel, timeFromStr);
+						}
+						strcat(infoLabel, "\n");
+
+						strcat(infoLabel, toStr);
+						if (strcmp(timeToStr,"")) {
+
+							strcat(infoLabel, " - ");
+							strcat(infoLabel, timeToStr);
+						}
+						strcat(infoLabel, "\n");
+
+						if (strcmp(alternateStr,"")) {
+
+							strcat(infoLabel, "<i>");
+							strcat(infoLabel, alternateStr);
+							strcat(infoLabel, "</i>\n");
+						}
+
+						if (strcmp(infoStr,"")) {
+
+							strcat(infoLabel, "<b>");
+							strcat(infoLabel, infoStr);
+							strcat(infoLabel, "</b>\n");
+						}
+
+						infoLabel[strlen(infoLabel) -1] = '\0';
+
+						line = gtk_button_new();
+						gtk_button_set_focus_on_click(GTK_BUTTON(line), FALSE);
+						gtk_button_set_relief(GTK_BUTTON(line), GTK_RELIEF_NONE);
+
+						lineHbox = gtk_hbox_new(FALSE, 0);
+						gtk_container_add(GTK_CONTAINER(line), lineHbox);
+						lineText = gtk_label_new(NULL);
+						gtk_label_set_justify(GTK_LABEL(lineText), GTK_JUSTIFY_FILL);
+						gtk_label_set_markup(GTK_LABEL(lineText), infoLabel);
+						gtk_box_pack_start(GTK_BOX(lineHbox), lineText, FALSE, TRUE, 10);
+
+						gtk_box_pack_start(GTK_BOX(data->currentDetailsMainVbox), line, FALSE, TRUE, 0);
+						gtk_box_pack_start(GTK_BOX(data->currentDetailsMainVbox), gtk_hseparator_new(), FALSE, TRUE, 0);
+
 					}
 					if (!strcmp(cur_node->properties->children->content, "connectiondetails")) {
 
@@ -657,23 +752,77 @@ void loadDetails(char* targetUrl, AppData *data) {
 							if (!strcmp(sub_node->name, "span")) {
 
 								pricingStr = sub_node->children->content;
+								pricingStr += 1;
 							}
 						}
 
+						removeLineBreaks(departureStr);
+						removeLineBreaks(arrivalStr);
+						removeLineBreaks(durationStr);
+						removeLineBreaks(infoStr);
+						removeLineBreaks(pricingStr);
+
 						g_debug("From: %s", departureStr);
 						g_debug("To: %s", arrivalStr);
 						g_debug("Duration: %s", durationStr);
 						g_debug("Info: %s", infoStr);
 						g_debug("Price: %s", pricingStr);
 
+						GtkTreeIter    iter;
+						char infoLabel[1000] = "";
+
+						strcat(infoLabel, "<b>");
+						strcat(infoLabel, departureStr);
+						strcat(infoLabel, "</b>");
+
+						if (strcmp(departureStr, arrivalStr)) {
+
+							strcat(infoLabel, _(" to "));
+							strcat(infoLabel, "<b>");
+							strcat(infoLabel, arrivalStr);
+							strcat(infoLabel, "</b>");
+						}
+						strcat(infoLabel, "\n");
+
+						strcat(infoLabel, "<b>");
+						strcat(infoLabel, _("Duration: "));
+						strcat(infoLabel, "</b>");
+						strcat(infoLabel, durationStr);
+						strcat(infoLabel, "\n");
+
+						if (strcmp(infoStr,"")) {
+
+							strcat(infoLabel, "<b>");
+							strcat(infoLabel, infoStr);
+							strcat(infoLabel, "</b>\n");
+						}
+
+						if (strcmp(pricingStr,"")) {
+
+							strcat(infoLabel, "<i>");
+							strcat(infoLabel, pricingStr);
+							strcat(infoLabel, "</i>\n");
+						}
+
+						infoLabel[strlen(infoLabel) -1] = '\0';
+
+						line = gtk_button_new();
+						gtk_button_set_focus_on_click(GTK_BUTTON(line), FALSE);
+						gtk_button_set_relief(GTK_BUTTON(line), GTK_RELIEF_NONE);
+
+						lineHbox = gtk_hbox_new(FALSE, 0);
+						gtk_container_add(GTK_CONTAINER(line), lineHbox);
+						lineText = gtk_label_new(NULL);
+						gtk_label_set_justify(GTK_LABEL(lineText), GTK_JUSTIFY_FILL);
+						gtk_label_set_markup(GTK_LABEL(lineText), infoLabel);
+						gtk_box_pack_start(GTK_BOX(lineHbox), lineText, FALSE, TRUE, 10);
+
+						gtk_box_pack_start(GTK_BOX(data->currentDetailsMainVbox), line, FALSE, TRUE, 0);
+						gtk_box_pack_start(GTK_BOX(data->currentDetailsMainVbox), gtk_hseparator_new(), FALSE, TRUE, 0);
 					}
 				}
 			}
 
-			gtk_tree_view_set_model (GTK_TREE_VIEW (data->currentDetailsSelector), GTK_TREE_MODEL(data->currentDetailsModelStore));
-
-			gtk_container_add(GTK_CONTAINER (data->currentDetailsPannableArea), data->currentDetailsSelector);
-
 			gtk_container_add(GTK_CONTAINER (data->detailsWindow), data->currentDetailsPannableArea);
 
 			gtk_widget_show_all(GTK_WIDGET(data->detailsWindow));
@@ -681,31 +830,6 @@ void loadDetails(char* targetUrl, AppData *data) {
 	}
 }
 
-/**
- * Function to handle the selection of an entry of the results page
- */
-void resultSelected(GtkTreeView * selector, GtkTreePath *arg1, AppData *data) {
-
-
-	gchar *current_selection;
-
-	gint num = (gtk_tree_path_get_indices (arg1))[0];
-
-
-	char urlLabel[100];
-
-	sprintf(urlLabel, "URL%d", num + 1);
-
-	current_selection = g_object_get_data(G_OBJECT(data->currentResultSelector), urlLabel);
-
-	/*
-	hildon_banner_show_information(GTK_WIDGET(data->window), NULL,
-			current_selection);
-	*/
-
-	loadDetails(current_selection, data);
-}
-
 /**
  * Parse the search result itself
  */
@@ -960,34 +1084,16 @@ void parseSearchResult(char *returnData, AppData * data) {
 		/*Get the root element node */
 		root_element = xmlDocGetRootElement(doc);
 
-		GtkCellRenderer     *renderer;
 
-		data->currentResultSelector = hildon_gtk_tree_view_new(HILDON_UI_MODE_EDIT);
 		data->currentResultPannableArea = hildon_pannable_area_new();
 
-		renderer = gtk_cell_renderer_text_new ();
-		gtk_tree_view_insert_column_with_attributes (GTK_TREE_VIEW (data->currentResultSelector),
-												   -1,
-												   "Title",
-												   renderer,
-												   "markup", 0,
-												   NULL);
+		data->currentResultMainVbox = gtk_vbox_new(FALSE, 5);
 
-		data->currentResultModelStore = gtk_list_store_new (1, G_TYPE_STRING);
+	    hildon_pannable_area_add_with_viewport(HILDON_PANNABLE_AREA (data->currentResultPannableArea), GTK_WIDGET (data->currentResultMainVbox));
 
 		getResultValues(root_element->children, data);
 
-		gtk_tree_view_set_model (GTK_TREE_VIEW (data->currentResultSelector), GTK_TREE_MODEL(data->currentResultModelStore));
-
-		/*
-		 * Details page not yet finished so disable
-		g_signal_connect (data->currentResultSelector, "hildon-row-tapped",  G_CALLBACK (resultSelected), data);
-		*/
-
-		gtk_container_add(GTK_CONTAINER (data->currentResultPannableArea), data->currentResultSelector);
-
-		gtk_box_pack_start(GTK_BOX (vbox), data->currentResultPannableArea, TRUE,
-							TRUE, 0);
+		gtk_box_pack_start(GTK_BOX (vbox), data->currentResultPannableArea, TRUE, TRUE, 0);
 
 
 	}
@@ -1292,6 +1398,16 @@ void getStationsByName(GtkWidget *w, AppData * data) {
 	gchar *searchFor = hildon_entry_get_text(
 			HILDON_ENTRY(data->currentSearchTextBox));
 
+	if (strlen(searchFor) == 0) {
+
+		hildon_banner_show_information(
+				GTK_WIDGET(data->window),
+				NULL,
+				_("Please enter a Station name"));
+		return;
+	}
+
+
 	/* Start Curl search */
 	CURL *curl;
 	CURLcode res;
@@ -1430,29 +1546,195 @@ void getStationsByName(GtkWidget *w, AppData * data) {
 		gtk_widget_destroy(data->currentDialog);
 		data->currentDialog = NULL;
 	}
+}
 
+static void onGPSError(LocationGPSDControl *control,gint error, AppData *data)
+{
+	hildon_banner_show_information(GTK_WIDGET(data->window), NULL,
+							_("Error with GPS"));
+
+	hildon_gtk_window_set_progress_indicator(GTK_WINDOW(data->currentDialog), 0);
+	location_gpsd_control_stop(data->currentGPSControl);
+}
+
+/**
+ * Gets current latitude and longitude and sends a request.
+ * after a response received parses it and displays a selector dialog.
+ */
+static void onGPSLocation(LocationGPSDevice *device, AppData * data)
+{
+	if (!device)
+		return;
+
+	if (device->fix) {
+		if (device->fix->fields & LOCATION_GPS_DEVICE_LATLONG_SET) {
+
+			hildon_gtk_window_set_progress_indicator(GTK_WINDOW(data->currentDialog), 0);
+			location_gpsd_control_stop(data->currentGPSControl);
+
+			g_object_unref(device);
+			g_object_unref(data->currentGPSControl);
+
+			int poslat = device->fix->latitude * 1000000;
+			int poslong = device->fix->longitude * 1000000;
+
+			/* Start Curl search */
+			CURL *curl;
+			CURLcode res;
+			char buf[50000];
+			struct CURLData curldata;
+
+			curldata.buf = buf;
+			curldata.size = 50000;
+			curldata.pos = 0;
+
+			curl = curl_easy_init();
+			if (curl) {
+
+				char url[1000];
+
+				int	urlCount =
+								sprintf(
+										url,
+										"http://mobile.bahn.de/bin/mobil/query.exe/dox?rt=1&use_realtime_filter=1&stationNear=1&look_x=%d&look_y=%d&performLocating=2&tpl=stopsnear&look_maxdist=1000&look_stopclass=1023&n=1",
+										poslong, poslat);
+				url[urlCount] = '\0';
+
+				curl_easy_setopt(curl, CURLOPT_URL, url);
+				curl_easy_setopt(curl, CURLOPT_WRITEFUNCTION, &copyDataBuffer);
+				curl_easy_setopt(curl, CURLOPT_WRITEDATA, &curldata);
+				curl_easy_setopt(curl, CURLOPT_FAILONERROR, 1);
+				curl_easy_setopt(curl, CURLOPT_TIMEOUT, 150L);
+				curl_easy_setopt(curl, CURLOPT_CONNECTTIMEOUT, 15L);
+
+				res = curl_easy_perform(curl);
+
+				if (res != CURLE_OK) {
+
+					hildon_banner_show_information(
+							GTK_WIDGET(data->window),
+							NULL,
+							_("An error occured while requesting data over the internet."));
+					gtk_widget_hide(data->currentDialog);
+					gtk_widget_destroy(data->currentDialog);
+					data->currentDialog = NULL;
+
+					g_debug("CURL_ERROR: %d", res);
+
+					return;
+				}
+
+				char *dataBoxFrom = strstr(curldata.buf, "<div class=\"overview\">");
+
+				if (dataBoxFrom) {
+
+					char *dataBoxTo = strstr(dataBoxFrom, "<p class=\"prio2links\"");
+					int count = dataBoxTo - dataBoxFrom;
+					char dataBox[count + 6];
+					strncpy(dataBox, dataBoxFrom, count);
+
+					dataBox[count] = '\0';
+
+					strcat(dataBox, "</div>");
+
+					g_debug("DATA %s", dataBox);
+
+					xmlDoc *doc = NULL;
+					xmlNode *root_element = NULL;
+
+					/*parse the file and get the DOM */
+					doc = xmlReadDoc(dataBox, NULL, NULL, 0);
+
+					/*Get the root element node */
+					root_element = xmlDocGetRootElement(doc);
+					xmlNode *cur_node = NULL;
+					xmlNode *tmp_node = NULL;
+
+					if (data->currentSelector) {
+
+						gtk_widget_destroy(data->currentSelector);
+					}
+
+					data->currentSelector = hildon_touch_selector_new_text();
+
+					int num = 0;
+
+					for (cur_node = root_element->children; cur_node; cur_node
+							= cur_node->next) {
+
+						if (!strcmp(cur_node->name, "div")) {
+
+							for (tmp_node = cur_node->children; tmp_node; tmp_node
+									= tmp_node->next) {
+
+								if (!strcmp(tmp_node->name, "a")) {
+
+									num++;
+									/* g_debug("-- %s", tmp_node->children->content); */
+									hildon_touch_selector_append_text(
+											HILDON_TOUCH_SELECTOR (data->currentSelector),
+											tmp_node->children->content);
+								}
+							}
+						}
+					}
+
+					if (num > 0) {
+
+						if (data->currentTmpPicker) {
+
+							gtk_widget_destroy(data->currentTmpPicker);
+						}
+
+						data->currentTmpPicker = hildon_picker_button_new(
+								HILDON_SIZE_AUTO, HILDON_BUTTON_ARRANGEMENT_VERTICAL);
+						hildon_button_set_title(HILDON_BUTTON (data->currentTmpPicker),
+								_("Stations found"));
+
+						g_signal_connect (G_OBJECT (data->currentTmpPicker), "value-changed",
+								G_CALLBACK (selectionChanged), data);
+
+						hildon_picker_button_set_selector(
+								HILDON_PICKER_BUTTON (data->currentTmpPicker),
+								HILDON_TOUCH_SELECTOR (data->currentSelector));
+
+						gtk_button_clicked(GTK_BUTTON (data->currentTmpPicker));
+
+						gtk_widget_hide(data->currentDialog);
+						gtk_widget_destroy(data->currentDialog);
+						data->currentDialog = NULL;
+
+					} else {
+
+						hildon_banner_show_information(GTK_WIDGET(data->window), NULL,
+								_("No Stations in reach found."));
+					}
+				}
+			}
+
+		}
+	}
 }
 
 /**
  * Callback if the "GPS" button on the search for station dialog is pressed.
- * Gets current latitude and longitude and sends a request.
- * after a response received parses it and displays a selector dialog.
+ * Starts the GPS Request and waits until a position is received.
  */
 void getByGPS(GtkWidget *w, AppData * data) {
 
-	LocationGPSDControl *control = location_gpsd_control_get_default();
+	data->currentGPSControl = location_gpsd_control_get_default();
 	LocationGPSDevice *device = g_object_new(LOCATION_TYPE_GPS_DEVICE, NULL);
 
-	g_object_set(G_OBJECT(control), "preferred-method",
+	g_object_set(G_OBJECT(data->currentGPSControl), "preferred-method",
 			LOCATION_METHOD_USER_SELECTED, "preferred-interval",
 			LOCATION_INTERVAL_DEFAULT, NULL);
 
-	location_gpsd_control_start(control);
+	hildon_gtk_window_set_progress_indicator(GTK_WINDOW(data->currentDialog), 1);
 
-	int poslat = device->fix->latitude * 1000000;
-	int poslong = device->fix->longitude * 1000000;
+	g_signal_connect(device, "changed", G_CALLBACK(onGPSLocation), data);
+	g_signal_connect(data->currentGPSControl, "error-verbose", G_CALLBACK(onGPSError), data);
 
-	location_gpsd_control_stop(control);
+	location_gpsd_control_start(data->currentGPSControl);
 
 	if ((device->fix->latitude > 0) || (device->fix->latitude < 0)) {
 
@@ -1460,160 +1742,12 @@ void getByGPS(GtkWidget *w, AppData * data) {
 
 		hildon_banner_show_information(GTK_WIDGET(data->window), NULL,
 				_("Error with location service"));
+
+		hildon_gtk_window_set_progress_indicator(GTK_WINDOW(data->currentDialog), 0);
+		location_gpsd_control_stop(data->currentGPSControl);
+
 		return;
 	}
-
-	/* Start Curl search */
-	CURL *curl;
-	CURLcode res;
-	char buf[50000];
-	struct CURLData curldata;
-
-	curldata.buf = buf;
-	curldata.size = 50000;
-	curldata.pos = 0;
-
-	curl = curl_easy_init();
-	if (curl) {
-
-		char url[1000];
-
-		int	urlCount =
-						sprintf(
-								url,
-								"http://mobile.bahn.de/bin/mobil/query.exe/dox?rt=1&use_realtime_filter=1&stationNear=1&look_x=%d&look_y=%d&performLocating=2&tpl=stopsnear&look_maxdist=1000&look_stopclass=1023&n=1",
-								poslong, poslat);
-		url[urlCount] = '\0';
-
-		curl_easy_setopt(curl, CURLOPT_URL, url);
-		curl_easy_setopt(curl, CURLOPT_WRITEFUNCTION, &copyDataBuffer);
-		curl_easy_setopt(curl, CURLOPT_WRITEDATA, &curldata);
-		curl_easy_setopt(curl, CURLOPT_FAILONERROR, 1);
-		curl_easy_setopt(curl, CURLOPT_TIMEOUT, 150L);
-		curl_easy_setopt(curl, CURLOPT_CONNECTTIMEOUT, 15L);
-
-		res = curl_easy_perform(curl);
-
-		if (res != CURLE_OK) {
-
-			hildon_banner_show_information(
-					GTK_WIDGET(data->window),
-					NULL,
-					_("An error occured while requesting data over the internet."));
-			gtk_widget_hide(data->currentDialog);
-			gtk_widget_destroy(data->currentDialog);
-			data->currentDialog = NULL;
-
-			g_debug("CURL_ERROR: %d", res);
-
-			return;
-		}
-
-		char *dataBoxFrom = strstr(curldata.buf, "<div class=\"overview\">");
-
-		if (dataBoxFrom) {
-
-			char *dataBoxTo = strstr(dataBoxFrom, "<p class=\"prio2links\"");
-			int count = dataBoxTo - dataBoxFrom;
-			char dataBox[count + 6];
-			strncpy(dataBox, dataBoxFrom, count);
-
-			dataBox[count] = '\0';
-
-			strcat(dataBox, "</div>");
-
-			g_debug("DATA %s", dataBox);
-
-			xmlDoc *doc = NULL;
-			xmlNode *root_element = NULL;
-
-			/*parse the file and get the DOM */
-			doc = xmlReadDoc(dataBox, NULL, NULL, 0);
-
-			/*Get the root element node */
-			root_element = xmlDocGetRootElement(doc);
-			xmlNode *cur_node = NULL;
-			xmlNode *tmp_node = NULL;
-
-			if (data->currentSelector) {
-
-				gtk_widget_destroy(data->currentSelector);
-			}
-
-			data->currentSelector = hildon_touch_selector_new_text();
-
-			int num = 0;
-
-			for (cur_node = root_element->children; cur_node; cur_node
-					= cur_node->next) {
-
-				if (!strcmp(cur_node->name, "div")) {
-
-					for (tmp_node = cur_node->children; tmp_node; tmp_node
-							= tmp_node->next) {
-
-						if (!strcmp(tmp_node->name, "a")) {
-
-							num++;
-							/* g_debug("-- %s", tmp_node->children->content); */
-							hildon_touch_selector_append_text(
-									HILDON_TOUCH_SELECTOR (data->currentSelector),
-									tmp_node->children->content);
-						}
-					}
-				}
-			}
-
-			if (num > 0) {
-
-				if (data->currentTmpPicker) {
-
-					gtk_widget_destroy(data->currentTmpPicker);
-				}
-
-				data->currentTmpPicker = hildon_picker_button_new(
-						HILDON_SIZE_AUTO, HILDON_BUTTON_ARRANGEMENT_VERTICAL);
-				hildon_button_set_title(HILDON_BUTTON (data->currentTmpPicker),
-						_("Stations found"));
-
-				g_signal_connect (G_OBJECT (data->currentTmpPicker), "value-changed",
-						G_CALLBACK (selectionChanged), data);
-
-				hildon_picker_button_set_selector(
-						HILDON_PICKER_BUTTON (data->currentTmpPicker),
-						HILDON_TOUCH_SELECTOR (data->currentSelector));
-
-				gtk_button_clicked(GTK_BUTTON (data->currentTmpPicker));
-
-				gtk_widget_hide(data->currentDialog);
-				gtk_widget_destroy(data->currentDialog);
-				data->currentDialog = NULL;
-
-				/*
-
-				 char *label[200];
-
-				 sprintf(label, "Found %d Stations in reach", num);
-
-				 hildon_banner_show_information(GTK_WIDGET(data->window), NULL, label);
-
-				 */
-
-			} else {
-
-				/*
-				 char *label[200];
-
-				 sprintf(label, "lat = %d, long = %d", poslat, poslong);
-
-				 hildon_banner_show_information(GTK_WIDGET(data->window), NULL, label);
-				 */
-
-				hildon_banner_show_information(GTK_WIDGET(data->window), NULL,
-						_("No Stations in reach found."));
-			}
-		}
-	}
 }
 
 /**
@@ -1676,30 +1810,18 @@ static gint dbus_callback(const gchar *interface, const gchar *method,
  */
 static void aboutClicked(HildonButton * button, AppData *data) {
 
-	GtkWidget *aboutDialog;
-	GtkWidget *aboutLabel;
-	GtkWidget *versionLabel;
+	char infoLabel[1000] = "";
 
-	aboutDialog = gtk_dialog_new();
-	gtk_window_set_transient_for(GTK_WINDOW (aboutDialog),
-			GTK_WINDOW (data->window));
-	gtk_window_set_title(GTK_WINDOW (aboutDialog),
-			_("About"));
+	strcat(infoLabel, "<b>About Fahrplan</b>\n");
+	strcat(infoLabel, "by smurfy (maemo@smurfy.de)\n");
+	strcat(infoLabel, "Version ");
+	strcat(infoLabel, APP_VER);
 
-	aboutLabel = gtk_label_new(("by smurfy (maemo@smurfy.de)"));
+	hildon_banner_show_information_with_markup(
+			GTK_WIDGET(data->window),
+			NULL,
+			infoLabel);
 
-	gtk_box_pack_start(GTK_BOX (GTK_DIALOG(aboutDialog)->vbox),	aboutLabel, TRUE, TRUE, 0);
-
-	char labelText[255];
-	int count = sprintf(labelText, "Version %s", APP_VER);
-	labelText[count] = '\0';
-
-	versionLabel = gtk_label_new(labelText);
-
-	gtk_box_pack_start(GTK_BOX (GTK_DIALOG(aboutDialog)->vbox),	versionLabel, TRUE, TRUE, 0);
-
-	gtk_widget_show_all(GTK_WIDGET (aboutDialog));
-	gtk_dialog_run(GTK_DIALOG (aboutDialog));
 }
 
 /**
@@ -1868,14 +1990,12 @@ void initGui(AppData *data) {
 
 	gtk_widget_show_all(GTK_WIDGET ( data->window ));
 
-
 /*
 	hildon_button_set_value(HILDON_BUTTON(data->fromButton),
 					"Berlin-Tempelhof");
 
 	hildon_button_set_value(HILDON_BUTTON(data->toButton),
 						"Feldkirchen Blindenschuhle, Neuwied");
-
 */
 }
 
